import inspect
import pathlib
import re

from collections import defaultdict
from typing import Any, Dict, List, Optional, Set, Tuple, Type, get_type_hints

import faker.proxy

from faker import Factory, Faker
from faker.config import AVAILABLE_LOCALES, PROVIDERS

BUILTIN_MODULES_TO_IGNORE = ["builtins"]
GENERIC_MANGLE_TYPES_TO_IGNORE = ["builtin_function_or_method", "mappingproxy"]
MODULES_TO_FULLY_QUALIFY = ["datetime"]


imports: Dict[str, Optional[Set[str]]] = defaultdict(lambda: None)
imports["collections"] = {"OrderedDict"}
imports["json"] = {"encoder"}
imports["typing"] = {"Callable", "Collection", "TypeVar"}
imports["uuid"] = {"UUID"}
imports["enum"] = {"Enum"}
imports["faker.typing"] = {"*"}
imports["faker.generator"] = {"Generator"}


def get_module_and_member_to_import(cls: Type, locale: Optional[str] = None) -> Tuple[str, str]:
    cls_name = getattr(cls, "__name__", getattr(cls, "_name", str(cls)))
    module, member = cls.__module__, cls_name
    if cls_name is None:
        qualified_type = re.findall(r"([a-zA-Z_0-9]+)\.([a-zA-Z_0-9]+)", str(cls))
        if len(qualified_type) > 0:
            if imports[qualified_type[0][0]] is None or qualified_type[0][1] not in imports[qualified_type[0][0]]:
                module, member = qualified_type[0]
        else:
            unqualified_type = re.findall(r"[^\.a-zA-Z0-9_]([A-Z][a-zA-Z0-9_]+)[^\.a-zA-Z0-9_]", " " + str(cls) + " ")
            if len(unqualified_type) > 0 and unqualified_type[0] != "NoneType":
                cls_str = str(cls).replace(".en_US", "").replace("faker.", ".")
                if "<class '" in cls_str:
                    cls_str = cls_str.split("'")[1]
                if locale is not None:
                    cls_str = cls_str.replace("." + locale, "")

                if imports[cls_str] is None or unqualified_type[0] not in imports[cls_str]:
                    module, member = cls_str, unqualified_type[0]
    if module in MODULES_TO_FULLY_QUALIFY:
        member = None
    return module, member


seen_funcs = set()
seen_vars = set()


class UniqueMemberFunctionsAndVariables:
    def __init__(self, cls: type, funcs: Dict[str, Any], vars: Dict[str, Any]):
        global seen_funcs, seen_vars
        self.cls = cls
        self.funcs = funcs
        for func_name in seen_funcs:
            self.funcs.pop(func_name, None)
        seen_funcs = seen_funcs.union(self.funcs.keys())

        self.vars = vars
        for var_name in seen_vars.union(seen_funcs):
            self.vars.pop(var_name, None)
        seen_vars = seen_vars.union(self.vars.keys())


def get_member_functions_and_variables(cls: object, include_mangled: bool = False) -> UniqueMemberFunctionsAndVariables:
    members = [
        (name, value)
        for (name, value) in inspect.getmembers(cls)
        if ((include_mangled and name.startswith("__")) or not name.startswith("_"))
    ]
    funcs: Dict[str, Any] = {}
    vars: Dict[str, Any] = {}
    for name, value in members:
        attr = getattr(cls, name, None)
        if attr is not None and (inspect.isfunction(attr) or inspect.ismethod(attr)):
            funcs[name] = value
        elif inspect.isgetsetdescriptor(attr) or inspect.ismethoddescriptor(attr):
            # I haven't implemented logic
            # for generating descriptor signatures yet
            continue
        elif not include_mangled or type(value).__name__ not in GENERIC_MANGLE_TYPES_TO_IGNORE:
            vars[name] = value

    return UniqueMemberFunctionsAndVariables(cls, funcs, vars)


classes_and_locales_to_use_for_stub: List[Tuple[object, str]] = []
for locale in AVAILABLE_LOCALES:
    for provider in PROVIDERS:
        if provider == "faker.providers":
            continue
        prov_cls, _, _ = Factory._find_provider_class(provider, locale)
        classes_and_locales_to_use_for_stub.append((prov_cls, locale))

all_members: List[Tuple[UniqueMemberFunctionsAndVariables, str]] = [
    (get_member_functions_and_variables(cls), locale) for cls, locale in classes_and_locales_to_use_for_stub
] + [(get_member_functions_and_variables(Faker, include_mangled=True), None)]

# Use the accumulated seen_funcs and seen_vars to remove all variables that have the same name as a function somewhere
overlapping_var_names = seen_vars.intersection(seen_funcs)
for mbr_funcs_and_vars, _ in all_members:
    for var_name_to_remove in overlapping_var_names:
        mbr_funcs_and_vars.vars.pop(var_name_to_remove, None)

# list of tuples. First elem of tuple is the signature string,
#  second is the comment string,
#  third is a boolean which is True if the comment precedes the signature
signatures_with_comments: List[Tuple[str, str, bool]] = []

for mbr_funcs_and_vars, locale in all_members:
    for func_name, func_value in mbr_funcs_and_vars.funcs.items():
        sig = inspect.signature(func_value)
        hints = get_type_hints(func_value)
        ret_annot_module = getattr(sig.return_annotation, "__module__", None)
        if sig.return_annotation not in [None, inspect.Signature.empty, prov_cls.__name__] and ret_annot_module not in [
            None,
            *BUILTIN_MODULES_TO_IGNORE,
        ]:
            module, member = get_module_and_member_to_import(sig.return_annotation, locale)
            if module is not None:
                if imports[module] is None:
                    imports[module] = set() if member is None else {member}
                elif member is not None:
                    imports[module].add(member)

        new_parms = []
        for key, parm_val in sig.parameters.items():
            new_parm = parm_val
            annotation = hints.get(key, new_parm.annotation)
            if parm_val.default is not inspect.Parameter.empty:
                new_parm = parm_val.replace(default=...)
            if annotation is not inspect.Parameter.empty and annotation.__module__ not in BUILTIN_MODULES_TO_IGNORE:
                module, member = get_module_and_member_to_import(annotation, locale)
                if module is not None:
                    if imports[module] is None:
                        imports[module] = set() if member is None else {member}
                    elif member is not None:
                        imports[module].add(member)
            new_parms.append(new_parm)

        sig = sig.replace(parameters=new_parms)
        sig_str = str(sig).replace("Ellipsis", "...").replace("NoneType", "None").replace("~", "")
        for module in imports.keys():
            if module in MODULES_TO_FULLY_QUALIFY:
                continue
            sig_str = sig_str.replace(f"{module}.", "")

        decorator = ""
        if list(sig.parameters)[0] == "cls":
            decorator = "@classmethod\n"
        elif list(sig.parameters)[0] != "self":
            decorator = "@staticmethod\n"
        comment = inspect.getdoc(func_value)
        signatures_with_comments.append(
            (f"{decorator}def {func_name}{sig_str}: ...", None if comment == "" else comment, False)
        )

signatures_with_comments_as_str = []
for sig, comment, is_preceding_comment in signatures_with_comments:
    if comment is not None and is_preceding_comment:
        signatures_with_comments_as_str.append(f"# {comment}\n    {sig}")
    elif comment is not None:
        sig_without_final_ellipsis = sig.strip(" .")
        signatures_with_comments_as_str.append(
            sig_without_final_ellipsis + '\n    """\n    ' + comment.replace("\n", "\n    ") + '\n    """\n    ...'
        )
    else:
        signatures_with_comments_as_str.append(sig)


def get_import_str(module: str, members: Optional[Set[str]]) -> str:
    if members is None or len(members) == 0:
        return f"import {module}"
    else:
        return f"from {module} import {', '.join(members)}"


imports_block = "\n".join([get_import_str(module, names) for module, names in imports.items()])
member_signatures_block = "    " + "\n    ".join(
    [sig.replace("\n", "\n    ") for sig in signatures_with_comments_as_str]
)

body = f"""# This file is auto-generated by generate_stubs.py.
# Please do not edit this file directly.

{imports_block}

class Faker:
{member_signatures_block}
"""

faker_proxy_path = pathlib.Path(inspect.getfile(faker.proxy))
stub_file_path = faker_proxy_path.with_name("proxy.pyi").resolve()
with open(stub_file_path, "w", encoding="utf-8") as fh:
    fh.write(body)



import inspect

from unittest import mock
from unittest.mock import MagicMock

from faker.config import DEFAULT_LOCALE
from faker.sphinx.docstring import DEFAULT_SAMPLE_SIZE, DEFAULT_SEED, ProviderMethodDocstring, Sample


class TestProviderMethodDocstring:
    def test_what_is_not_method(self):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="not_a_method",
            name="name",
            obj=MagicMock,
            options=MagicMock(),
            lines=MagicMock(),
        )
        assert docstring.skipped

    def test_name_is_not_dotted_path_to_provider_method(self):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.sphinx.docstring.ProviderMethodDocString._parse",
            obj=MagicMock,
            options=MagicMock(),
            lines=MagicMock(),
        )
        assert docstring.skipped

    def test_name_is_dotted_path_to_base_provider_method(self):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.BaseProvider.bothify",
            obj=MagicMock,
            options=MagicMock(),
            lines=MagicMock(),
        )
        assert not docstring.skipped
        assert docstring._method == "bothify"
        assert docstring._locale == DEFAULT_LOCALE

    def test_name_is_dotted_path_to_standard_provider_method(self):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.barcode.Provider.upc_a",
            obj=MagicMock,
            options=MagicMock(),
            lines=MagicMock(),
        )
        assert not docstring.skipped
        assert docstring._method == "upc_a"
        assert docstring._locale == DEFAULT_LOCALE

    def test_name_is_dotted_path_to_localized_provider_method(self):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.automotive.en_PH.Provider.protocol_license_plate",
            obj=MagicMock,
            options=MagicMock(),
            lines=MagicMock(),
        )
        assert not docstring.skipped
        assert docstring._method == "protocol_license_plate"
        assert docstring._locale == "en_PH"

    @mock.patch("faker.sphinx.docstring.logger.warning")
    def test_log_warning(self, mock_logger_warning):
        path = inspect.getfile(MagicMock)
        name = "faker.providers.color.Provider"
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name=name,
            obj=MagicMock,
            options=MagicMock(),
            lines=MagicMock(),
        )
        docstring._log_warning("Test Warning 1")
        docstring._log_warning("Test Warning 2")

        assert docstring._log_prefix == f"{path}:docstring of {name}: WARNING:"

        calls = mock_logger_warning.call_args_list
        assert len(calls) == 2

        # 1st call to logger.warning
        args, kwargs = calls[0]
        assert len(args) == 1
        assert not kwargs
        assert args[0] == f"{path}:docstring of {name}: WARNING: Test Warning 1"

        # 2nd call to logger.warning
        args, kwargs = calls[1]
        assert len(args) == 1
        assert not kwargs
        assert args[0] == f"{path}:docstring of {name}: WARNING: Test Warning 2"

    def test_stringify_results(self, faker):
        class TestObject:
            def __repr__(self):
                return "abcdefg"

        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.BaseProvider.bothify",
            obj=MagicMock,
            options=MagicMock(),
            lines=[],
        )
        results = [
            "",  # Empty string
            "'",  # Single quote literal (escaped)
            "'",  # Single quote literal (unescaped)
            '"',  # Double quote literal (unescaped)
            '"',  # Double quote literal (escaped)
            "aa\taaaaa\r\n",  # String containing \t, \r, \n
            b"abcdef",  # Bytes object
            True,  # Booleans
            False,
            None,  # None types
            [1, 2, 3, 4, 5],  # Other non-primitives
            (1, 2, 3, 4, 5),
            {1: 2, 2: 3, 3: 4, 4: 5},
            faker.uuid4(cast_to=None),
            TestObject(),
        ]
        output = [docstring._stringify_result(result) for result in results]
        assert output == [
            "''",  # Ends up as '' when printed
            '"\'"',  # Ends up as "'" when printed
            '"\'"',  # Ends up as "'" when printed
            "'\"'",  # Ends up as '"' when printed
            "'\"'",  # Ends up as '"' when printed
            "'aa\\taaaaa\\r\\n'",  # Ends up as 'aa\\taaaaa\\r\\n' when printed
            "b'abcdef'",  # Ends up as b'abcdef' when printed
            "True",  # Ends up as True when printed
            "False",  # Ends up as False when printed
            "None",  # Ends up as None when printed
            "[1, 2, 3, 4, 5]",  # Ends up using object's __repr__
            "(1, 2, 3, 4, 5)",
            "{1: 2, 2: 3, 3: 4, 4: 5}",
            "UUID('e3e70682-c209-4cac-a29f-6fbed82c07cd')",
            "abcdefg",
        ]

    @mock.patch.object(ProviderMethodDocstring, "_log_warning")
    def test_parsing_empty_lines(self, mock_log_warning):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.BaseProvider.bothify",
            obj=MagicMock,
            options=MagicMock(),
            lines=[],
        )
        assert not docstring.skipped
        assert len(docstring._samples) == 1
        assert docstring._samples[0] == Sample(DEFAULT_SAMPLE_SIZE, DEFAULT_SEED, "")

    @mock.patch.object(ProviderMethodDocstring, "_log_warning")
    def test_parsing_single_line_non_sample(self, mock_log_warning):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.BaseProvider.bothify",
            obj=MagicMock,
            options=MagicMock(),
            lines=["lorem"],
        )
        assert not docstring.skipped
        assert len(docstring._samples) == 1
        assert docstring._samples[0] == Sample(DEFAULT_SAMPLE_SIZE, DEFAULT_SEED, "")

    @mock.patch.object(ProviderMethodDocstring, "_log_warning")
    def test_parsing_single_line_valid_sample(self, mock_log_warning):
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.BaseProvider.bothify",
            obj=MagicMock,
            options=MagicMock(),
            lines=[":sample: a=1"],
        )
        assert not docstring.skipped
        assert docstring._samples == [Sample(5, 0, "a=1")]

    @mock.patch.object(ProviderMethodDocstring, "_log_warning")
    def test_parsing_multiple_lines(self, mock_log_warning):
        lines = [
            "lorem",  # No-op, not a sample line
            ":sample:",  # Valid, default sample count, default seed, empty kwargs, 1st in expected
            ":sample 10 2000:",  # Invalid, size and seed must be specified as "keyword arguments"
            ":sample 10 seed=1000:",  # Invalid, size and seed must be specified as "keyword arguments"
            ":sample size=10 1000:",  # Invalid, size and seed must be specified as "keyword arguments"
            ":sample size=0:",  # Invalid, sample count cannot be zero
            ":sample size=100:",  # Valid, 100 samples, default seed, empty kwargs, 2nd in expected
            ":sample size=0100:",  # Invalid, leading zeroes are not allowed
            ":sampler",  # Invalid, starts with ":sample" but will not pass validation
            ":sample :",  # No-op, must be ":sample:" verbatim
            ":sample seed=4761:",  # Valid, default sample count, seed value of 4761
            "",  # but line break was detected, so sample parsing stops here
            "ipsum",  # No-op, not a sample line
            ":sample sede=123",  # Invalid, seed misspelled
            ":sample size=4 seed=100:",  # Valid, will reset to 5 samples, seed value of 100, empty kwargs, the 4th
            ":sample seed=103 size=104:",  # Invalid, "seed" kwarg must come after "size" kwarg
            ":sample: a=1, b=2",  # Valid, default count and seed with kwargs, the 5th
            ":sample size=2222: a=2, b=1",  # Valid, 2222 samples, default seed, and with kwargs, the 6th
            ":sample 11 12:",  # Invalid, seed value must be set with "seed=" prefix
            ":sample seed=3333: d=3",  # Valid, default count, seed value of 3333, with kwargs, the 7th
            ":sample size=3333 seed=2222: c=1",  # Valid, 3333 samples, seed value of 2222, with kwargs, the 8th
            ":sample size=10 seed=10:",  # Valid 9th, 10 samples, seed value of 10, with kwargs
            "   arg1=1,",  # and will continue reading the next few lines
            '   arg2="val2",arg3="val3",',  # and will prettify (missing whitespace after comma)
            " arg4=4   ,    arg5=5,",  # and will remove excess whitespaces here
            ' arg6="ar  g6",',  # but not if whitespaces are within double quotes
            "       arg7='   ar  g 7',",  # or within single quotes
            '    arg8="aaa,aaa"',  # and will not prettify commas within quotes
            ":sample size=20 seed=3456:",  # Valid 10th, 20 samples, seed value of 3456, with kwargs
            'arg1="val1,val1,val1",',  # and this is very similar to previous sample
            'arg2="val2",',  # and it is ok not to have leading whitespaces in continuation lines
            'arg3="val3    val3",',  # and it is ok to have a trailing comma after the last kwarg
        ]

        expected_output = [
            Sample(DEFAULT_SAMPLE_SIZE, DEFAULT_SEED, ""),  # 1st sample parsed
            Sample(100, DEFAULT_SEED, ""),  # 2nd sample parsed
            Sample(DEFAULT_SAMPLE_SIZE, 4761, ""),  # 3rd sample parsed
            Sample(5, 100, ""),  # 4th sample parsed
            Sample(DEFAULT_SAMPLE_SIZE, DEFAULT_SEED, "a=1, b=2"),  # 5th sample parsed
            Sample(2222, DEFAULT_SEED, "a=2, b=1"),  # 6th sample parsed
            Sample(DEFAULT_SAMPLE_SIZE, 3333, "d=3"),  # 7th sample parsed
            Sample(3333, 2222, "c=1"),  # 8th sample parsed
            Sample(  # 9th sample parsed
                10,
                10,
                'arg1=1, arg2="val2", arg3="val3", arg4=4, arg5=5, arg6="ar  g6", arg7=\'   ar  g 7\', arg8="aaa,aaa"',
            ),
            Sample(  # 10th sample parsed
                20,
                3456,
                'arg1="val1,val1,val1", arg2="val2", arg3="val3    val3",',
            ),
        ]
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.BaseProvider.bothify",
            obj=MagicMock,
            options=MagicMock(),
            lines=lines,
        )
        assert not docstring.skipped
        assert docstring._samples == expected_output

    @mock.patch.object(ProviderMethodDocstring, "_log_warning")
    def test_end_to_end_sample_generation(self, mock_warning, faker):
        non_sample_lines = ["lorem", "ipsum", "dolor", "sit", "amet"]
        valid_sample_lines = [
            ":sample 1234jdbvhjdbygdvbhxjhx",  # Will fail during sample section processing, 1st log warning
            ":sample: invalid_arg='value'",  # Will fail during sample generation, 2nd log warning
            ":sample size=3 seed=1000: text='???###'",  # 1st sample generation
            ":sample: number=100**100**100",  # Will fail SampleCodeValidator validation, 3rd log warning
            ":sample seed=3210: letters='abcde'",  # 2nd sample generation
            ":sample size=10 seed=1: abcd='abcd'",  # Will fail during sample generation, 4th log warning
            ":sample size=20 seed=1234: text='???###', ",  # 3rd sample generation
            "         letters='abcde'",
        ]
        lines = non_sample_lines + valid_sample_lines
        docstring = ProviderMethodDocstring(
            app=MagicMock(),
            what="method",
            name="faker.providers.BaseProvider.bothify",
            obj=MagicMock,
            options=MagicMock(),
            lines=lines,
        )

        output = docstring.lines[len(non_sample_lines) :]
        assert output[0] == ":examples:"

        # 1st sample generation
        faker.seed_instance(1000)
        assert output[1] == ""
        assert output[2] == ">>> Faker.seed(1000)"
        assert output[3] == ">>> for _ in range(5):"
        assert output[4] == "...     fake.bothify(text='???###')"
        assert output[5] == "..."
        for i in range(6, 11):
            assert output[i] == docstring._stringify_result(faker.bothify(text="???###"))

        # 2nd sample generation
        faker.seed_instance(3210)
        assert output[11] == ""
        assert output[12] == ">>> Faker.seed(3210)"
        assert output[13] == ">>> for _ in range(5):"
        assert output[14] == "...     fake.bothify(letters='abcde')"
        assert output[15] == "..."
        for i in range(16, 21):
            assert output[i] == docstring._stringify_result(faker.bothify(letters="abcde"))

        # 3rd sample generation
        faker.seed_instance(1234)
        assert output[21] == ""
        assert output[22] == ">>> Faker.seed(1234)"
        assert output[23] == ">>> for _ in range(20):"
        assert output[24] == "...     fake.bothify(text='???###', letters='abcde')"
        assert output[25] == "..."
        for i in range(26, 46):
            assert output[i] == docstring._stringify_result(faker.bothify(text="???###", letters="abcde"))

        calls = mock_warning.call_args_list
        assert len(calls) == 4

        # 1st call to _log_warning
        args, kwargs = calls[0]
        assert len(args) == 1
        assert not kwargs
        assert args[0] == "The section `:sample 1234jdbvhjdbygdvbhxjhx` is malformed and will be discarded."

        # 2nd call to _log_warning
        args, kwargs = calls[1]
        assert len(args) == 1
        assert not kwargs
        assert args[0] == "Sample generation failed for method `bothify` with arguments `invalid_arg='value'`."

        # 3rd call to _log_warning
        args, kwargs = calls[2]
        assert len(args) == 1
        assert not kwargs
        assert args[0] == (
            "Invalid code elements detected. Sample generation will be skipped for "
            "method `bothify` with arguments `number=100**100**100`."
        )

        # 4th call to _log_warning
        args, kwargs = calls[3]
        assert len(args) == 1
        assert not kwargs
        assert args[0] == "Sample generation failed for method `bothify` with arguments `abcd='abcd'`."



from unittest import mock

from faker.sphinx.validator import SampleCodeValidator


class TestSampleCodeValidator:
    @mock.patch("faker.sphinx.validator.ast.parse")
    def test_ast_parser_called_in_eval_mode(self, mock_ast_parse):
        command = "variable.method()"
        mock_ast_parse.assert_not_called()
        SampleCodeValidator(command)
        mock_ast_parse.assert_called_with(command, mode="eval")

    def test_invalid_syntax(self):
        validator = SampleCodeValidator("[T___T)")
        assert validator.errors

    def test_single_instance_of_variable_access(self):
        validator = SampleCodeValidator("variable1")
        assert not validator.errors

        validator = SampleCodeValidator("[variable2]")
        assert not validator.errors

        validator = SampleCodeValidator("[variable1, variable1]")
        assert validator.errors

        validator = SampleCodeValidator("[variable1, variable2]")
        assert validator.errors

    def test_ordereddict_variable_access_exception(self):
        validator = SampleCodeValidator("[OrderedDict]")
        assert not validator.errors

        validator = SampleCodeValidator("[OrderedDict, variable1]")
        assert not validator.errors

        validator = SampleCodeValidator("[OrderedDict, OrderedDict, OrderedDict, OrderedDict]")
        assert not validator.errors

        validator = SampleCodeValidator("[OrderedDict, OrderedDict, OrderedDict, OrderedDict, variable1]")
        assert not validator.errors

        validator = SampleCodeValidator("[OrderedDict, variable1, variable2]")
        assert validator.errors

        validator = SampleCodeValidator("[OrderedDict2, variable1]")
        assert validator.errors

    def test_single_instance_of_attribute_access(self):
        validator = SampleCodeValidator("variable.attr1")
        assert not validator.errors

        validator = SampleCodeValidator("variable.attr1.attr2")
        assert validator.errors

    def test_single_instance_of_method_or_function_call(self):
        validator = SampleCodeValidator("variable.method1()")
        assert not validator.errors

        validator = SampleCodeValidator("variable.method1().method2()")
        assert validator.errors

    def test_ordereddict_constructor_exception(self):
        validator = SampleCodeValidator("OrderedDict()")
        assert not validator.errors

        validator = SampleCodeValidator("variable.method1(OrderedDict())")
        assert not validator.errors

        validator = SampleCodeValidator("variable.method1(OrderedDict2())")
        assert validator.errors

        validator = SampleCodeValidator("variable.method1(OrderedDict(), OrderedDict(), OrderedDict())")
        assert not validator.errors

        validator = SampleCodeValidator("variable.method1().method2(OrderedDict())")
        assert validator.errors

    def test_allowed_literal_types(self):
        commands = [
            "variable.method(12345)",
            'variable.method("12345")',
            'variable.method(b"12345")',
            "variable.method([1, 2, 3, 4, 5])",
            "variable.method((1, 2, 3, 4, 5))",
            "variable.method({1, 2, 3, 4, 5})",
            'variable.method({"value": 12345})',
            "variable.method(True)",
            "variable.method(False)",
            "variable.method(None)",
        ]

        for command in commands:
            validator = SampleCodeValidator(command)
            assert not validator.errors

    def test_prohibited_literal_types(self):
        commands = ["variable.method(...)"]
        commands.append('f"{variable}"')

        for command in commands:
            validator = SampleCodeValidator(command)
            assert validator.errors

    def test_prohibited_operations(self):
        commands = [
            # Unary Operations
            "+variable",
            "-variable",
            "not variable",
            "~variable",
            # Mathematical Operations
            "variable + 5",
            "variable - 5",
            "variable * 5",
            "variable / 5",
            "variable // 5",
            "variable % 5",
            "variable ** 5",
            # Bitwise Operations
            "variable << 5",
            "variable >> 5",
            "variable | 5",
            "variable ^ 5",
            "variable & 5",
            # Boolean Operations
            "variable and True",
            "variable or True",
            # Comparisons
            "variable is None",
            "variable is not None",
            "variable in [1, 2, 3, 4, 5]",
            "variable not in [1, 2, 3, 4, 5]",
            "variable == 5",
            "variable != 5",
            "variable < 5",
            "variable <= 5",
            "variable > 5",
            "variable >= 5",
        ]

        for command in commands:
            validator = SampleCodeValidator(command)
            assert validator.errors

    def test_other_prohibited_expressions(self):
        commands = [
            "variable if True else 1",
            "variable[1]",
            "variable[1:3]",
            "variable[1:3, 2]",
            "[True for _ in [1, 2, 3]]",
            "(True for _ in [1, 2, 3])",
            "{True for _ in [1, 2, 3]}",
            "{str(_): _ for _ in [1, 2, 3]}",
            "lambda x: x",
        ]

        for command in commands:
            validator = SampleCodeValidator(command)
            assert validator.errors



from typing import Tuple

from faker.typing import SexLiteral

from .. import Provider as PassportProvider


class Provider(PassportProvider):
    def passport_owner(self, gender: SexLiteral = "M") -> Tuple[str, str]:
        given_name, surname = super().passport_owner(gender)
        return surname, given_name



#!/usr/bin/env python

from pathlib import Path

from setuptools import find_packages, setup

here = Path(__file__).resolve().parent
README = (here / "README.rst").read_text(encoding="utf-8")
VERSION = (here / "VERSION").read_text(encoding="utf-8").strip()

excluded_packages = ["docs", "tests", "tests.*"]


# this module can be zip-safe if the zipimporter implements iter_modules or if
# pkgutil.iter_importer_modules has registered a dispatch for the zipimporter.
try:
    import pkgutil
    import zipimport

    zip_safe = (
        hasattr(zipimport.zipimporter, "iter_modules")
        or zipimport.zipimporter in pkgutil.iter_importer_modules.registry.keys()
    )
except AttributeError:
    zip_safe = False

setup(
    name="Faker",
    version=VERSION,
    description="Faker is a Python package that generates fake data for you.",
    long_description=README,
    entry_points={
        "console_scripts": ["faker=faker.cli:execute_from_command_line"],
        "pytest11": ["faker = faker.contrib.pytest.plugin"],
    },
    classifiers=[
        # See https://pypi.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Testing",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    keywords="faker fixtures data test mock generator",
    author="joke2k",
    author_email="joke2k@gmail.com",
    url="https://github.com/joke2k/faker",
    project_urls={
        "Bug Tracker": "https://github.com/joke2k/faker/issues",
        "Changes": "https://github.com/joke2k/faker/blob/master/CHANGELOG.md",
        "Documentation": "http://faker.rtfd.org/",
        "Source Code": "https://github.com/joke2k/faker",
    },
    license="MIT License",
    packages=find_packages(exclude=excluded_packages),
    package_data={
        "faker": ["py.typed", "proxy.pyi"],
    },
    platforms=["any"],
    zip_safe=zip_safe,
    python_requires=">=3.8",
    install_requires=[
        "python-dateutil>=2.4",
        "typing_extensions",
    ],
)
